## This module handles the configuration file for Sympa.

package Conf;

use Log;
use Language;

require Exporter;
use Carp;

@ISA = qw(Exporter);
@EXPORT = qw(%Conf);

## RCS identification.
#my $id = '@(#)$Id: Conf.pm,v 1.4 1998/12/22 13:52:38 sympa Exp $';

my @valid_options = qw(
   avg archives bye clean_delay_queue 
   clean_delay_queueauth clean_delay_queuemod 
   config create_list db_host db_name db_passwd 
   db_type db_user default_list_priority email expire 
   help help_advanced home host info lang listmaster lists lists_header 
   lists_footer log_socket_type max_size maxsmtp message_footer
   message_header msgcat nrcpt owner_priority pidfile 
   queue queueauth queuedigest queueexpire queuemod queueoutgoing
   reminder removed request_list request_priority scenari sendmail sleep 
   sort stats subscribers sympa_priority syslog umask welcome
);
my %valid_options = ();
map { $valid_options{$_}++; } @valid_options;

my %Default_Conf = 
    ('home'    => '/home/sympa/expl',
     'lists'   => 'lists',
     'help'    => 'helpfile',
     'help_advanced'    => 'helpfile.advanced',
     'umask'   => '027',
     'syslog'  => 'LOCAL1',
     'nrcpt'   => 25,
     'avg'     => 10,
     'maxsmtp' => 20,
     'sendmail'=> '/usr/sbin/sendmail',
     'host'    => undef,            ## No default value => required field
     'email'   => 'sympa',
     'pidfile' => '/home/sympa/sympa.pid',
     'msgcat'  => '/home/sympa/nls',
     'scenari' => '/home/sympa/bin/scenari',
     'sort'    => 'fr,ca,be,ch,uk,edu,*,com',
     'queue'   => undef, ## No default value => required field
     'queuedigest'=> '/home/sympa/queuedigest',
     'queuemod'   => '/home/sympa/queuemoderation',
     'queueexpire'=> '/home/sympa/queueexpire',
     'queueauth'  => '/home/sympa/queueauth',
     'queueoutgoing'  => '/dev/null',
     'sleep'      => 5,
     'clean_delay_queue'    => 1,
     'clean_delay_queuemod' => 10,
     'clean_delay_queueauth' => 3,
     'log_socket_type'      => 'inet',
     'lists_header'         => 'lists.header',
     'lists_footer'         => 'lists.footer',
     'welcome'            => 'welcome',
     'subscribers'        => 'subscribers',
     'info'               => 'info',
     'archives'           => 'archives' ,
     'config'             => 'config' ,
     'stats'              => 'stats' ,
     'message_footer'     => 'message.footer' ,
     'message_header'     => 'message.header' ,
     'bye'                => 'bye',
     'removed'            => 'removed',
     'expire'             => 'expire',
     'reminder'           => 'reminder',
     'return_path'        => 'owner',
     'db_type' => undef,
     'db_name' => 'sympa',
     'db_host' => undef,
     'db_user' => undef, 
     'db_passwd'  => undef,
     'listmaster' => undef,
     'default_list_priority' => 5,
     'sympa_priority' => 1,
     'request_priority' => 0,
     'owner_priority' => 9,
     'lang' => 'us',
     'max_size' => 5242880,
     'request_list' => 'listmaster',
     'create_list' => 'listmaster'
   );
   
%Conf = ();

## Loads and parses the configuration file. Reports errors if any.
sub load {
    my $config = shift;
    my $line_num = 0;
    my $config_err = 0;
    my($i, %o);
    
    ## Open the configuration file or return and read the lines.
    unless (open(IN, $config)) {
	printf STDERR  "Unable to open %s: %s\n", $config, $!;
	return undef;
    }
    while (<IN>) {
	$line_num++;
	next if (/^\s*$/o || /^[\#\;]/o);
	if (/^(\S+)\s+(\S+|\`.*\`)\s*$/io) {
	    my($keyword, $value) = ($1, $2);
	    ##  'tri' is a synonime for 'sort' (for compatibily with old versions)
	    $keyword = 'sort' if ($keyword eq 'tri');
	    ## Special case: `command`
	    if ($value =~ /^\`(.*)\`$/) {
		$value = qx/$1/;
		chomp($value);
	    }
	    $o{$keyword} = [ $value, $line_num ];
	}else {
	    printf STDERR Msg(1, 3, "Malformed line %d: %s"), $config, $_;
	    $config_err++;
	}
    }
    close(IN);
    
    ## Check if we have unknown values.
    foreach $i (sort keys %o) {
	next if ($valid_options{$i});
	printf STDERR  "Line %d, unknown field: %s\n", $o{$i}[1], $i;
	$config_err++;
    }
    ## Do we have all required values ?
    foreach $i (keys %valid_options) {
	unless (exists $o{$i} or exists $Default_Conf{$i}) {
	    printf "Required field not found: %s\n", $i;
	    $config_err++;
	    next;
	}
	$Conf{$i} = $o{$i}[0] || $Default_Conf{$i};
    }
    
    my $p = 1;
    foreach (split(/,/, $Conf{'sort'})) {
	$Conf{'poids'}{$_} = $p++;
    }
    $Conf{'poids'}{'*'} = $p if ! $Conf{'poids'}{'*'};
    
    if ($config_err) {
	return undef;
    }
    
    $Conf{'sympa'} = "$Conf{'email'}\@$Conf{'host'}";
    $Conf{'request'} = "$Conf{'email'}-request\@$Conf{'host'}";
    
    return 1;
}

## Check a few files
sub checkfiles {
    my $config_err = 0;
    
    unless (-x $Conf{'sendmail'}) {
	do_log('err', "File %s does not exist or is not executable", $Conf{'sendmail'});
	$config_err++;
    }
    unless (-r $Conf{'help'}) {
	do_log('err', "Helpfile %s does not exist or is not readable", $Conf{'help'});
	$config_err++;
    }
    
    my $qdir = $Conf{'queue'};
    unless (-d $qdir && -r $qdir && -w $qdir && -x $qdir) {
	do_log('err', "Queue directory %s does not exist or has incorrect permissions", $qdir);
	$config_err++;
    }
    my $ddir = $Conf{'queuedigest'};
    unless (-d $ddir && -r $ddir && -w $ddir && -x $ddir) {
	do_log('err',"Queue directory %s does not exist or has incorrect permissions", $ddir);
	$config_err++;
    }
    my $mdir = $Conf{'queuemod'};
    unless (-d $mdir && -r $mdir && -w $mdir && -x $mdir) {
	do_log('err', "Queue directory %s does not exist or has incorrect permissions", $mdir);
	$config_err++;
    }
    $mdir = $Conf{'queueexpire'};
    unless (-d $mdir && -r $mdir && -w $mdir && -x $mdir) {
	do_log('err', "Queue directory #%s# does not exist or has incorrect permissions", $mdir);
	    $config_err++;
    }
    $mdir = $Conf{'queueauth'};
    unless (-d $mdir && -r $mdir && -w $mdir && -x $mdir) {
	do_log('err', "Queue directory %s does not exist or has incorrect permissions", $mdir);
	$config_err++;
    } 
    return undef if ($config_err);
    return 1;
}

## Packages must return true.
1;
